/*****************************************************************************
 * RS232 Pendant Driver for VistaCNC P2 Pendant
 *
 * Copyright(c) 2012 by:
 *      Ray Livingston
 *      20 Braemoor Drive
 *      Bonny Doon, CA  95060
 *
 * May be used freely for non-commercial purposes, providing copyright notice
 * is nto removed, and proper attribution is given.  This is provided as-is,
 * aboslutely no warranty is expressed or implied.  Use at your own risk.
 *
 * Operation:
 *
 * A message is sent whenever any input changes.  The message format is:
 *
 * Message format:  abcd
 *      a = Processed Action State => One of:
 *              I(dle)
 *              E(Stop) - Higher priority than all others
 *              H(old) -> Press and hold Action for > 200mSec
 *              L(ong Hold) -> Double Click, and hold on second press
 *              R(elease) -> Action released after Hold or Long Hold
 *              D(oubleClick) -> Time between all Action changes < 200 mSec
 *      b = Mode Select, '0'-'5' (Mode 5 is blocked in DEBUG mode)
 *      c = Axis Select, 'X', 'Y', 'Z', 'A', 'B', 'C'
 *      d = Raw Action Button State, '0'-'1'
 *
 *      Each change of mode or axis will result in a single message, indicating
 *      the new mode or axis.
 *
 *      Pressing E-Stop will result in a single message indicating a Processed
 *      Action State of E(Stop).  Note that this is in addition to the external
 *      E-Stop switch action.
 *
 *      Releasing E-Stop will result in a single message indicating a Processed
 *      Action State of I(dle).  Note that this is in addition to the external
 *      E-Stop switch action.
 *
 *      Pressing the Action button can generate a series of messages as
 *      follows:
 *
 *      Quick Action Press and Release:
 *          1) Message indicating Action Button State = 1 on press
 *          2) Message indicating Action Button State = 0 on release
 *
 *      Action Press and Hold (Hold):
 *          1) Message indicating Action Button State = 1 on press
 *          2) Message indicating Processed Action State = H(old)
 *          3) Message indicating Action Button State = 0 on release
 *          4) Message indicating Processed Action State = R(elease)
 *
 *      Action Double-Click:
 *          1) Message indicating Action Button State = 1 on first press
 *          2) Message indicating Action Button State = 0 on first release
 *          3) Message indicating Action Button State = 1 on second press
 *          4) Message indicating Action Button State = 0 on second release
 *          5) Message indicating Processed Action State = D(ouble-click)
 *
 *      Action Double-Click And Hold on Second Click (Long Hold):
 *          1) Message indicating Action Button State = 1 on first press
 *          2) Message indicating Action Button State = 0 on first release
 *          3) Message indicating Action Button State = 1 on second press
 *          4) Message indicating Processed Action State = L(ong Hold)
 *          5) Message indicating Action Button State = 0 on second release
 *          6) Message indicating Processed Action State = R(elease)
 *
 *      Note that mode and axis change messages are suppressed whenever Action
 *      is depressed, or there is a possuble double-click in process.  The
 *      final mode and/or axis will be reported once Action is released.
 *
 * Hardware Interfaces:
 *
 * I/Os are documented in code - mostly in Pendant.h
 *
 * Cable Pinout (Made from DB-9 RS232 extender cable):
 *
 * 1 = Unused
 * 2 = VCC      (Red wire)
 * 3 = RxD      (Org wire)
 * 4 = MPG-A    (Yellow wire)
 * 5 = TxD      (Green wire)
 * 6 = MPG-B    (Blue wire)
 * 7 = EStop    (Purple wire)
 * 8 = EStop    (Gray wire)
 * 9 = GND      (Black wire)
 *
 * Serial Baud rate is 9600
 * CPU clock = 24 MHz
 *
 * Note the following hardware modifications were made to enable the above
 * functionality:
 *
 * 1) Action Switch was moved from MClR/RE3 to D-/RC4 (since USB is no longer
 *    used).  This allows Action to be used in Debug mode (it was originally
 *    on the MCLR pin.
 * 2) RS232 wires were soldered directly to pins RC7/RC6
 * 3) PIC was re-connected to MPG A-/B- (currently not used...)
 * 4) External MPG connections are connected to MPG A+/B+
 * 5) External E-Stop is connected to second pole of E-Stop switch
 *
 *****************************************************************************/


/*****************************************************************************
 * Definitions - Must come before #includes
 *****************************************************************************/
//#define TIMERBLINK    // Blink LED once/second
#define SERIALBLINK     // Blink LED when serial message sent
#define BLINKTIME   50  // On-time for LED

#include "p18f4455.h"
#include "Pendant.h"
#include "Timer.h"
#include "SerialIO.h"


/*****************************************************************************
 * Prototypes
 *****************************************************************************/
void HighPriorityHandler(void);


/*****************************************************************************
 * PIC18F4455 Configuration Bit Settings
 *****************************************************************************/

    #pragma config PLLDIV   = 10
    #pragma config CPUDIV   = OSC1_PLL2
    #pragma config USBDIV   = 2
    #pragma config FOSC     = HSPLL_HS
    #pragma config FCMEN    = OFF
    #pragma config IESO     = OFF
    #pragma config PWRT     = OFF
    #pragma config BOR      = ON
    #pragma config BORV     = 3
    #pragma config VREGEN   = ON
    #pragma config WDT      = OFF
    #pragma config WDTPS    = 32768
    #pragma config MCLRE    = OFF
    #pragma config LPT1OSC  = OFF
    #pragma config PBADEN   = OFF
    #pragma config CCP2MX   = ON
    #pragma config STVREN   = ON
    #pragma config LVP      = OFF
    #pragma config ICPRT    = OFF
    #pragma config XINST    = OFF
    #pragma config CP0      = OFF
    #pragma config CP1      = OFF
    #pragma config CP2      = OFF
    #pragma config CPB      = OFF
    #pragma config CPD      = OFF
    #pragma config WRT0     = OFF
    #pragma config WRT1     = OFF
    #pragma config WRT2     = OFF
    #pragma config WRTB     = OFF
    #pragma config WRTC     = OFF
    #pragma config WRTD     = OFF
    #pragma config EBTR0    = OFF
    #pragma config EBTR1    = OFF
    #pragma config EBTR2    = OFF
    #pragma config EBTRB    = OFF


/*****************************************************************************
 * High Priority Interrupt Vector
 *****************************************************************************/
#pragma code high_vector=0x08
void HighPriorityVector(void)
{
    _asm GOTO HighPriorityHandler _endasm
}
#pragma code


/*****************************************************************************
 * High Priority Interrupt Handler
 *****************************************************************************/
#pragma interrupt HighPriorityHandler
void HighPriorityHandler  (void)
{
    if (PIR1bits.TMR2IF)
    {
        // Handle 1mSec timer tasks
        TimerHandler();

        // Handle Pendant-related functions
        DoPendantSwitches();
    }

    // Process RS232 RxData, if any
    if (PIR1bits.RCIF)
        RxHandler();

    // Process RS232 TxData, if any
    if (PIE1bits.TXIE && PIR1bits.TXIF)
        TxHandler();
}


/*****************************************************************************
 * main
 ****************************************************************************/
void main(void) {
unsigned char LastMode;
char LastAxis;
unsigned char LastAction;
char LastActionEvent;
unsigned char LastEStop;
volatile unsigned char x;

    // Disable USB
    UCONbits.USBEN = 0;
    UCFGbits.UTRDIS = 1;    // Must do this for RC4/5 to work as inputs!!
    SPPCONbits.SPPEN = 0;
    CCP1CONbits.CCP1M0 = 0;
    CCP1CONbits.CCP1M1 = 0;

    // Setup Pendant I/Os and data
    PendantOpen();

    // Setup TMR2 to generate 1mSec interrupts
    TimerOpen();

    // Setup Serial I/F
    SerialOpen();

    RCONbits.IPEN = 0; // Disable Interrupt Priority
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;

    LastMode = CurrentMode;
    LastAxis = CurrentAxis;
    LastAction = CurrentAction;
    LastActionEvent = CurrentActionEvent;
    LastEStop = CurrentEStop;

    while (1) {

        // Has something changed?
        if ((CurrentMode != LastMode) ||
            (CurrentAxis != LastAxis) ||
            (CurrentAction != LastAction) ||
            (CurrentActionEvent != LastActionEvent) ||
            (CurrentEStop != LastEStop)) {

            // Something has changed, so deal with it
            TxPut(CurrentEStop == 0 ? CurrentActionEvent : 'E');
            // This ActionEvent has been processed, so clear it
            ClearActionEvent();
            
            // Send message to host
            TxPut(CurrentMode + '0');
            TxPut(CurrentAxis < 3 ? CurrentAxis + 'X' : CurrentAxis + 'A' - 3);
            TxPut(CurrentAction + '0');
            TxPut('\r');
            TxPut('\n');

            LastMode = CurrentMode;
            LastAxis = CurrentAxis;
            LastAction = CurrentAction;
            LastActionEvent = CurrentActionEvent;
            LastEStop = CurrentEStop;
        }
    }
}


