#ifndef PENDANT_H
#define PENDANT_H

/*****************************************************************************
 * Pendant.h
 ****************************************************************************/

#include "Buffer.h"
#include "Debounce.h"


/*****************************************************************************
 * Definitions
 ****************************************************************************/

#define AxisBit0    PORTDbits.RD0
#define AxisBit1    PORTDbits.RD1
#define AxisBit2    PORTDbits.RD2
#define AxisBit3    PORTDbits.RD3
#define AxisBit4    PORTDbits.RD4
#define AxisBit5    PORTBbits.RB6

#define EStopBit0   PORTDbits.RD5

#define ModeBit0    PORTBbits.RB0
#define ModeBit1    PORTBbits.RB1
#define ModeBit2    PORTBbits.RB2
#define ModeBit3    PORTBbits.RB3
#define ModeBit4    PORTBbits.RB4
#define ModeBit5    PORTBbits.RB5

#define ActionBit0  PORTCbits.RC4

#define MPGBitA     PORTDbits.RD7
#define MPGBitB     PORTDbits.RD6

#define EStopLEDBit PORTAbits.RA4

#define	HOLD_TIMER_COUNT  200
#define LONG_HOLD_TIMER_COUNT 500

enum ActionStates {
    Idle = 0,
    FirstClick,
    FirstRelease,
    SecondClick,
    SecondRelease
};

enum DoubleClickStates {
    ACTION_IDLE	= 0,
    ACTION_FIRST_PRESS,
    ACTION_FIRST_RELEASE,
    ACTION_SECOND_PRESS,
    ACTION_HOLD,
    ACTION_LONG_HOLD,
    ACTION_WAIT,
    ACTION_DOUBLE_CLICK,
    ACTION_HOLD_RELEASE
};

/*****************************************************************************
 * Public Data
 ****************************************************************************/

unsigned char CurrentMode;
char CurrentAxis;
unsigned char CurrentAction;
char CurrentActionEvent;
unsigned char CurrentEStop;
unsigned char CurrentMPG;

unsigned char TmpMode;
unsigned char TmpAxis;


/*****************************************************************************
 * Private Data
 ****************************************************************************/

struct DebounceData ModeDB;
struct DebounceData AxisDB;
struct DebounceData ActionDB;
struct DebounceData EStopDB;
volatile struct Buffer MPGBuf;

int PendantDoubleClickState;
int HoldTimer;


/*****************************************************************************
 * PendantOpen - Initialize Pendant
 ****************************************************************************/
void PendantOpen(void)
{
    DebounceInit(&ModeDB, 0);
    DebounceInit(&AxisDB, 0);
    DebounceInit(&ActionDB, 0);
    DebounceInit(&EStopDB, 0);
    BufInit(&MPGBuf);
    CurrentMode = 0;
    CurrentAxis = 0;
    CurrentAction = 0;
    CurrentActionEvent = 'I';
    CurrentEStop = 0;
    CurrentMPG = 0;
    PendantDoubleClickState = ACTION_IDLE;
    HoldTimer = 0;

    TRISAbits.RA4 = 0;      // Configure EStopLED pin as output

    TRISBbits.TRISB0 = 1;   // Configure pins as inputs
    TRISBbits.TRISB1 = 1;
    TRISBbits.TRISB2 = 1;
    TRISBbits.TRISB3 = 1;
    TRISBbits.TRISB4 = 1;
    TRISBbits.TRISB5 = 1;
    TRISBbits.TRISB6 = 1;
    TRISBbits.TRISB7 = 1;
    INTCON2bits.RBPU = 0;   // Enable PORTB pullups

    TRISDbits.TRISD0 = 1;   // Configure pins as inputs
    TRISDbits.TRISD1 = 1;
    TRISDbits.TRISD2 = 1;
    TRISDbits.TRISD3 = 1;
    TRISDbits.TRISD4 = 1;
    TRISDbits.TRISD5 = 1;
    TRISDbits.TRISD6 = 1;
    TRISDbits.TRISD7 = 1;
    PORTEbits.RDPU = 1;     // Enable PORTD pullups

    EStopLEDBit = 0;
}


/*****************************************************************************
 * ReportAction - Report Updated Action State
 ****************************************************************************/
void ReportAction(int type)
{
    // Report Double-click or Hold event
    switch (type)
    {
        case ACTION_IDLE:
            // Report an Idle event
            CurrentActionEvent = 'I';
            break;

        case ACTION_HOLD:
            // Report a Hold event
            CurrentActionEvent = 'H';
            break;

        case ACTION_LONG_HOLD:
            // Report a Long Hold event
            CurrentActionEvent = 'L';
            break;

        case ACTION_HOLD_RELEASE:
            // Report a hold release event
            CurrentActionEvent = 'R';
            break;

        case ACTION_DOUBLE_CLICK:
            // Report a double-click event
            CurrentActionEvent = 'D';
            break;
    }
}


/*****************************************************************************
 * ClearActionEvent() - Clear CurrentActionEvent
 ****************************************************************************/
void ClearActionEvent(void)
{
    ReportAction(ACTION_IDLE);
}


/*****************************************************************************
 * DoDoubleClick - Double Click State Machine
 ****************************************************************************/
void DoDoubleClick(void) {
    // Decrement double-click timer
    if (HoldTimer)
        HoldTimer--;

    // Double-click state machine
    switch (PendantDoubleClickState) {

        case ACTION_IDLE:
            // Idle, so wait for first press
            HoldTimer = 0;
            if (CurrentAction) {
                // Got first press
                HoldTimer = HOLD_TIMER_COUNT;
                PendantDoubleClickState = ACTION_FIRST_PRESS;
            }
            break;

        case ACTION_FIRST_PRESS:
            // In first press, so wait for release
            if (!HoldTimer) {
                // Timed out, so this is a Hold
                PendantDoubleClickState = ACTION_HOLD;
                ReportAction(ACTION_HOLD);
            } else if (!CurrentAction) {
                // Release, so check for double-click
                HoldTimer = HOLD_TIMER_COUNT;
                PendantDoubleClickState = ACTION_FIRST_RELEASE;
            }
            break;

        case ACTION_FIRST_RELEASE:
            // In first release, so check for double-click
            if (!HoldTimer) {
                // Timed out, so just ignore
                PendantDoubleClickState = ACTION_IDLE;
            } else if (CurrentAction) {
                // Got second press
                HoldTimer = HOLD_TIMER_COUNT;
                PendantDoubleClickState = ACTION_SECOND_PRESS;
            }
            break;

        case ACTION_SECOND_PRESS:
            // In second press, so wait for release
            if (!HoldTimer) {
                // Timed out, so treat as LONG_HOLD
                ReportAction(ACTION_LONG_HOLD);
                PendantDoubleClickState = ACTION_HOLD;
            } else if (!CurrentAction) {
                // Got a double-click, so report it
                ReportAction(ACTION_DOUBLE_CLICK);
                // Return to idle
                HoldTimer = 0;
                PendantDoubleClickState = ACTION_IDLE;
            }
            break;

        case ACTION_HOLD:
        case ACTION_WAIT:
            // IN HOLD, so wait for release
            HoldTimer = 0;
            if (!CurrentAction) {
                // Got release, so return to idle
                if (PendantDoubleClickState == ACTION_HOLD)
                    ReportAction(ACTION_HOLD_RELEASE);
                PendantDoubleClickState = ACTION_IDLE;
            }
            break;
    }
}


/*****************************************************************************
 * ReMapBits - ReMap Mode/Axis Switch Bits to Integer Value
 ****************************************************************************/
char ReMapBits(char bits, char def)
{
    char ret = def;

    switch(bits)
    {
        case 0x001:
            ret = 0;
            break;
        case 0x02:
            ret = 1;
            break;
        case 0x04:
            ret = 2;
            break;
        case 0x08:
            ret = 3;
            break;
        case 0x10:
            ret = 4;
            break;
        case 0x20:
            ret = 5;
            break;
    }
    return ret;
}


/*****************************************************************************
 * DoPendantSwitches - Do pendant switch processing
 ****************************************************************************/
void DoPendantSwitches(void)
{
unsigned char ModeBits = 0;
unsigned char AxisBits = 0;
unsigned char ActionBit = 0;
unsigned char EStopBit = 0;
unsigned char MPGBits = 0;

    // Read switch inputs to xxxBits, decode, and debounce
    ModeBits = ((ModeBit5 ^ 1) << 5) |
               ((ModeBit4 ^ 1) << 4) |
               ((ModeBit3 ^ 1) << 3) |
               ((ModeBit2 ^ 1) << 2) |
               ((ModeBit1 ^ 1) << 1) |
               (ModeBit0 ^ 1);
    ModeBits = ReMapBits(ModeBits, ModeDB.Val);
    DebounceUpdate(&ModeDB, ModeBits);

    AxisBits = 
#ifndef __DEBUG
               ((AxisBit5 ^ 1) << 5) |
#endif
               ((AxisBit4 ^ 1) << 4) |
               ((AxisBit3 ^ 1) << 3) |
               ((AxisBit2 ^ 1) << 2) |
               ((AxisBit1 ^ 1) << 1) |
               (AxisBit0 ^ 1);
    AxisBits = ReMapBits(AxisBits, AxisDB.Val);
    DebounceUpdate(&AxisDB, AxisBits);

    ActionBit = ActionBit0;
    DebounceUpdate(&ActionDB, ActionBit);

    EStopBit = EStopBit0;
    DebounceUpdate(&EStopDB, EStopBit);

    // MPGBitA/B are inverted because we're on A-/B-
    MPGBits = ((MPGBitB ^ 1) << 1) | (MPGBitA ^ 1);
    if (MPGBits != CurrentMPG) {
        if (MPGBuf.Cnt == BUFSIZ)
            BufGet(&MPGBuf);
        BufPut(&MPGBuf, MPGBits);
    }
    CurrentMPG = MPGBits;

    // Save de-bounced switch states
    CurrentAction = ActionDB.Val;
    if ((CurrentAction == 0) && 
        (ActionDB.Cnt == 0) &&
        (PendantDoubleClickState == ACTION_IDLE))
    {
        // Block changes in Mode or Axis if Action is not released
        CurrentMode = ModeDB.Val;
        CurrentAxis = AxisDB.Val;
    }

    if (CurrentEStop != EStopDB.Val) {
        CurrentEStop = EStopDB.Val;
        EStopLEDBit = CurrentEStop;
    }

    // Do double-click processing
    DoDoubleClick();
}

#endif