#ifndef SERIALIO_H
#define SERIALIO_H

/*****************************************************************************
 * Serial.h - Interrupt-Driven Serial I/O
 ****************************************************************************/

#include "Buffer.h"
#include "usart.h"


/*****************************************************************************
 * Function Prototypes
 ****************************************************************************/

void TxHandler(void);
void RxHandler(void);


/*****************************************************************************
 * Definitions
 ****************************************************************************/

#define BAUD_DIVISOR	155

/*****************************************************************************
 * Private Data
 ****************************************************************************/

volatile struct Buffer TxBuf;
volatile struct Buffer RxBuf;


/*****************************************************************************
 * SerialOpen - Open Serial Port
 ****************************************************************************/
void SerialOpen(void) {
    BufInit(&RxBuf);
    BufInit(&TxBuf);

    PIE1bits.TXIE = 0;
    PIE1bits.RCIE = 0;

    BAUDCONbits.ABDOVF = 0;
    BAUDCONbits.RXDTP = 0;
    BAUDCONbits.TXCKP = 0;
    BAUDCONbits.BRG16 = 0;
    BAUDCONbits.WUE = 0;
    BAUDCONbits.ABDEN = 0;
    SPBRGH = 0;
    SPBRG = BAUD_DIVISOR;

    TXSTAbits.TX9 = 0;
    TXSTAbits.TXEN = 1;
    TXSTAbits.SYNC = 0;
    TXSTAbits.SENDB = 0;
    TXSTAbits.BRGH = 1;
    TXSTAbits.TX9D = 0;
    
    RCSTAbits.SPEN = 1;
    RCSTAbits.RX9 = 0;
    RCSTAbits.SREN = 0;
    RCSTAbits.CREN = 1;
    RCSTAbits.ADDEN = 0;

    PIE1bits.RCIE = 1;
}


/*****************************************************************************
 * SerialClose - Close Serial Port
 ****************************************************************************/
void SerialClose(void) {
    PIE1bits.TXIE = 0;
    PIE1bits.RCIE = 0;
    TXSTAbits.TXEN = 0;
    RCSTAbits.SPEN = 0;
}


/*****************************************************************************
 * TxPut - Add character to Tx Buffer.  Returns 0 if ok, non-zero if buffer
 * is full.
 ****************************************************************************/
char TxPut(char c) {
    char ret;

    ret = BufPut(&TxBuf, c);
    PIE1bits.TXIE = 1;
}


/*****************************************************************************
 * TxGet - Get next character from Tx Buffer.  Returns character if available,
 * zero if buffer is empty.
 ****************************************************************************/
char TxGet(void) {
    return BufGet(&TxBuf);
}


/*****************************************************************************
 * RxPut - Add character to Rx Buffer.  Returns 0 if ok, non-zero if buffer
 * is full.
 ****************************************************************************/
char RxPut(char c) {
    return BufPut(&RxBuf, c);
}


/*****************************************************************************
 * RxGet - Get next character from Rx Buffer.  Returns character if available,
 * zero if buffer is empty.
 ****************************************************************************/
char RxGet(void) {
    return BufGet(&RxBuf);
}


/*****************************************************************************
 * TxHandler - Tx Interrupt Handler
 *
 * Called by High Priority Interrupt Handler
 ****************************************************************************/
void TxHandler(void) {
    char c;

    c = TxGet();
    if (c)
        TXREG = c;  // Send the next character
    else
        PIE1bits.TXIE = 0; // Disable Tx Interrupt
#ifdef SERIALBLINK
    EStopLEDBit = CurrentEStop ^ 1;
    BlinkTimer = BLINKTIME;
#endif
}


/*****************************************************************************
 * RxHandler - Rx Interrupt Handler
 *
 * Called by High Priority Interrupt Handler
 ****************************************************************************/
void RxHandler(void) {
    char c = RCREG;
    RxPut(c);
    if (RxBuf.Cnt == BUFSIZ)
        RxBuf.Cnt = 0;
}

#endif